#!/bin/bash

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

cd -- "$(dirname "$0")" || exit 1
PATH+=:$PWD/app
FFMPEG_MAX_AGE=14
YTDL_MAX_AGE=1
YTDL_URL=https://github.com/yt-dlp/yt-dlp/releases/latest/download/yt-dlp
FFMPEG_URL=https://evermeet.cx/ffmpeg/get/zip
# python stuff
PATH+=:/usr/local/bin
PYTHON3_URL=https://www.python.org/ftp/python/3.10.1/python-3.10.1-macos11.pkg
PATH+=:/Library/Frameworks/Python.framework/Versions/3.10/bin
PATH+=:"/Applications/Python 3.10"

file_age_days() {
    local st_ctime=0
    eval "$(stat -s "$1")"
    local seconds=$(($(date +%s) - st_ctime))
    echo $((seconds / 86400))
}

## alphassl CA is missing on older macos
#choose_cacert() {
#    openssl s_client -host "$1" -port 443 </dev/null 2>/dev/null \
#        | grep AlphaSSL &>/dev/null \
#        && echo --cacert ca/alphassl-root.pem
#}

cat <<'EOF'

    *********************************************
    * yt-dlp helper (also for many other sites) *
    *********************************************
..........................................................
. Hint: always complete your answers with the Return key .
..........................................................
EOF

[ -d app ] || mkdir -v app

echo "checking presence of python3..."
if ! which python3; then
    echo "INFO: downloading python3..."
    if curl -L "$PYTHON3_URL" -o app/python3.pkg -#; then
        echo "Please enter your macOS user password to install:"
        sudo installer -pkg "$PWD/app/python3.pkg" -target /
        echo "INFO: successfully installed python3."
    else
        echo "WARNING: installation of python3 failed."
    fi
fi
if which "Install Certificates.command" 2>&1 >/dev/null; then
    echo "updating pip..."
    python3 -m pip install --upgrade pip
    echo "making sure certificates authorities are known..."
    "Install Certificates.command"
fi
if [ ! -x app/yt-dlp ] || [ "$(file_age_days app/yt-dlp)" -ge $YTDL_MAX_AGE ]; then
    echo "INFO: downloading yt-dlp"
    #  curl -L $(choose_cacert yt-dl.org)
    if curl -L "$YTDL_URL" -o app/yt-dlp -#; then
        chmod a+x app/yt-dlp
        echo "INFO: successfully downloaded yt-dlp"
    else
        echo "WARNING: download of yt-dlp failed."
    fi
fi
if [ ! -x app/yt-dlp ]; then
    echo "ERROR: yt-dlp is missing. Aborting."
    exit 1
fi
if [ ! -x app/ffmpeg ] || [ "$(file_age_days app/ffmpeg)" -ge $FFMPEG_MAX_AGE ]; then
    echo "INFO: downloading ffmpeg..."
    if curl -L "$FFMPEG_URL" -o app/ffmpeg.zip -#; then
        unzip -o app/ffmpeg.zip -d app
        chmod a+x app/ffmpeg
        echo "INFO: successfully downloaded ffmpeg."
    else
        echo "WARNING: download of ffmpeg failed."
    fi
fi
if [ ! -x app/ffmpeg ]; then
    echo "ERROR: ffmpeg is missing. Aborting."
    exit 1
fi

cat <<'EOF'

+----------------------------------------+
| Type 'a' for audio only or leave empty |
+----------------------------------------+
Your answer:
EOF

read -r MODE
case "$MODE" in
    a*|A*) OPTS="-x"; echo "=> audio mode: downloading only audio track" ;;
    *)     OPTS=""  ; echo "=> normal mode: downloading video and audio" ;;
esac

cat <<'EOF'

+-------------------------------------------------------------+
| Enter your login ID for videos needing login or leave empty |
+-------------------------------------------------------------+
Your answer:
EOF

read -r USERNAME
if [ -n "$USERNAME" ]; then
    OPTS+=' -u '$USERNAME
    echo "yt-dlp will ask for your password."
fi
[ -d downloaded ] || mkdir -v downloaded
cd downloaded || exit 1

cat <<'EOF'

**********************
* Ready to download! *
**********************
EOF

while :; do
    echo
    echo "Downloads are saved in: $PWD"
    cat <<'EOF'
Hint: if a URL contains the word "playlist", the entire playlist will be downloaded!
+---------------------------------------------------------------+
| Paste a video/playlist URL (right click); leave empty to exit |
+---------------------------------------------------------------+
Video URL:
EOF
    read -r URL
    echo
    case "$URL" in
        "") cat <<'EOF'
*************
* Good Bye! *
*************
EOF
            exit 0
            ;;
        *youtube*playlist*)
            yt-dlp -i $OPTS -o '%(playlist)s/%(playlist_index)s - %(title)s.%(ext)s' "$URL"
            ;;
        *) yt-dlp -i $OPTS "$URL" ;;
    esac
done
